@@ -13,6 +13,10 @@ WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 See the License for the specific language governing permissions and
 limitations under the License.
 """
+try:
+    import concurrent.futures as cf
+except ImportError:
+    pass
 import json
 from six.moves.urllib.parse import urlencode
 
@@ -28,6 +32,11 @@ def url_param_builder(param_dict):
     return "?{}".format(urlencode(param_dict))
 
 
+def calc_pages(limit, count):
+    """ Calculate number of pages required for full results set. """
+    return int(count / limit) + (limit % count > 0)
+
+
 class RequestError(Exception):
     """Basic Request Exception
 
@@ -49,9 +58,7 @@ class RequestError(Exception):
         req = message
 
         if req.status_code == 404:
-            message = "The requested url: {} could not be found.".format(
-                req.url
-            )
+            message = "The requested url: {} could not be found.".format(req.url)
         else:
             try:
                 message = "The request failed with code {} {}: {}".format(
@@ -104,8 +111,7 @@ class ContentError(Exception):
         req = message
 
         message = (
-            "The server returned invalid (non-json) data. Maybe not "
-            "a NetBox server?"
+            "The server returned invalid (non-json) data. Maybe not " "a NetBox server?"
         )
 
         super(ContentError, self).__init__(message)
@@ -139,8 +145,7 @@ class Request(object):
         token=None,
         private_key=None,
         session_key=None,
-        ssl_verify=True,
-        url=None,
+        threading=False,
     ):
         """
         Instantiates a new Request object
@@ -161,9 +166,23 @@ class Request(object):
         self.token = token
         self.private_key = private_key
         self.session_key = session_key
-        self.ssl_verify = ssl_verify
         self.http_session = http_session
         self.url = self.base if not key else "{}{}/".format(self.base, key)
+        self.threading = threading
+
+    def get_openapi(self):
+        """ Gets the OpenAPI Spec """
+        headers = {
+            "Content-Type": "application/json;",
+        }
+        req = self.http_session.get(
+            "{}docs/?format=openapi".format(self.normalize_url(self.base)),
+            headers=headers,
+        )
+        if req.ok:
+            return req.json()
+        else:
+            raise RequestError(req)
 
     def get_version(self):
         """ Gets the API version of NetBox.
@@ -178,11 +197,7 @@ class Request(object):
         headers = {
             "Content-Type": "application/json;",
         }
-        req = requests.get(
-            self.normalize_url(self.base),
-            headers=headers,
-            verify=self.ssl_verify,
-        )
+        req = self.http_session.get(self.normalize_url(self.base), headers=headers,)
         if req.ok:
             return req.headers.get("API-Version", "")
         else:
@@ -204,7 +219,6 @@ class Request(object):
                 "Content-Type": "application/x-www-form-urlencoded",
             },
             data=urlencode({"private_key": self.private_key.strip("\n")}),
-            verify=self.ssl_verify,
         )
         if req.ok:
             try:
@@ -222,9 +236,7 @@ class Request(object):
 
         return url
 
-    def _make_call(
-        self, verb="get", url_override=None, add_params=None, data=None
-    ):
+    def _make_call(self, verb="get", url_override=None, add_params=None, data=None):
         if verb in ("post", "put"):
             headers = {"Content-Type": "application/json;"}
         else:
@@ -238,13 +250,12 @@ class Request(object):
         params = {}
         if not url_override:
             if self.filters:
-                params = self.filters
+                params.update(self.filters)
             if add_params:
                 params.update(add_params)
 
         req = getattr(self.http_session, verb)(
-            url_override or self.url, headers=headers, verify=self.ssl_verify,
-            params=params, json=data
+            url_override or self.url, headers=headers, params=params, json=data
         )
 
         if req.status_code == 204 and verb == "post":
@@ -262,6 +273,19 @@ class Request(object):
         else:
             raise RequestError(req)
 
+    def concurrent_get(self, ret, page_size, page_offsets):
+        futures_to_results = []
+        with cf.ThreadPoolExecutor(max_workers=4) as pool:
+            for offset in page_offsets:
+                new_params = {"offset": offset, "limit": page_size}
+                futures_to_results.append(
+                    pool.submit(self._make_call, add_params=new_params)
+                )
+
+            for future in cf.as_completed(futures_to_results):
+                result = future.result()
+                ret.extend(result["results"])
+
     def get(self, add_params=None):
         """Makes a GET request.
 
@@ -285,10 +309,12 @@ class Request(object):
                     # passed in here because results from detail routes aren't
                     # paginated, thus far.
                     if first_run:
-                        req = self._make_call(add_params={
-                            "limit": req["count"],
-                            "offset": len(req["results"])
-                        })
+                        req = self._make_call(
+                            add_params={
+                                "limit": req["count"],
+                                "offset": len(req["results"]),
+                            }
+                        )
                     else:
                         req = self._make_call(url_override=req["next"])
                     first_run = False
@@ -297,6 +323,32 @@ class Request(object):
             else:
                 return req
 
+        def req_all_threaded(add_params):
+            if add_params is None:
+                # Limit must be 0 to discover the max page size
+                add_params = {"limit": 0}
+            req = self._make_call(add_params=add_params)
+            if isinstance(req, dict) and req.get("results") is not None:
+                ret = req["results"]
+                if req.get("next"):
+                    page_size = len(req["results"])
+                    pages = calc_pages(page_size, req["count"])
+                    page_offsets = [
+                        increment * page_size for increment in range(1, pages)
+                    ]
+                    if pages == 1:
+                        req = self._make_call(url_override=req.get("next"))
+                        ret.extend(req["results"])
+                    else:
+                        self.concurrent_get(ret, page_size, page_offsets)
+
+                return ret
+            else:
+                return req
+
+        if self.threading:
+            return req_all_threaded(add_params)
+
         return req_all()
 
     def put(self, data):
